@@ -38,6 +38,19 @@ TOP_LABEL = calc_label_from_name("top")
 DRAW_BYTES_LABEL = calc_label_from_name("draw_bytes() in ConjectureData")
 
 
+class ExtraInformation(object):
+    """A class for holding shared state on a ``ConjectureData`` that should
+    be added to the final ``ConjectureResult``."""
+
+    def __repr__(self):
+        return "ExtraInformation(%s)" % (
+            ", ".join(["%s=%r" % (k, v) for k, v in self.__dict__.items()]),
+        )
+
+    def has_information(self):
+        return bool(self.__dict__)
+
+
 class Status(IntEnum):
     OVERRUN = 0
     INVALID = 1
@@ -153,6 +166,27 @@ global_test_counter = 0
 MAX_DEPTH = 100
 
 
+@attr.s(slots=True)
+class ConjectureResult(object):
+    """Result class storing the parts of ConjectureData that we
+    will care about after the original ConjectureData has outlived its
+    usefulness."""
+
+    status = attr.ib()
+    interesting_origin = attr.ib()
+    buffer = attr.ib()
+    blocks = attr.ib()
+    examples = attr.ib()
+    has_discards = attr.ib()
+    output = attr.ib()
+    extra_information = attr.ib()
+
+    index = attr.ib(init=False)
+
+    def __attrs_post_init__(self):
+        self.index = len(self.buffer)
+
+
 class ConjectureData(object):
     @classmethod
     def for_buffer(self, buffer):
@@ -188,9 +222,11 @@ class ConjectureData(object):
         self.examples = []
         self.example_stack = []
         self.has_discards = False
+        self.__result = None
 
         top = self.start_example(TOP_LABEL)
         assert top.depth == 0
+        self.extra_information = ExtraInformation()
 
     def __repr__(self):
         return "ConjectureData(%s, %d bytes%s)" % (
@@ -199,6 +235,28 @@ class ConjectureData(object):
             ", frozen" if self.frozen else "",
         )
 
+    def as_result(self):
+        """Convert the result of running this test into
+        either an Overrun object or a ConjectureResult."""
+
+        assert self.frozen
+        if self.status == Status.OVERRUN:
+            return Overrun
+        if self.__result is None:
+            self.__result = ConjectureResult(
+                status=self.status,
+                interesting_origin=self.interesting_origin,
+                buffer=self.buffer,
+                examples=self.examples,
+                blocks=self.blocks,
+                has_discards=self.has_discards,
+                output=self.output,
+                extra_information=self.extra_information
+                if self.extra_information.has_information()
+                else None,
+            )
+        return self.__result
+
     def __assert_not_frozen(self, name):
         if self.frozen:
             raise Frozen("Cannot call %s on frozen ConjectureData" % (name,))
@@ -209,9 +267,6 @@ class ConjectureData(object):
         # that as depth 0 rather than depth 1.
         return len(self.example_stack) - 1
 
-    def all_block_bounds(self):
-        return [block.bounds for block in self.blocks]
-
     def note(self, value):
         self.__assert_not_frozen("note")
         if not isinstance(value, text_type):
