@@ -22,7 +22,7 @@ from collections import defaultdict
 import attr
 
 from hypothesis.internal.compat import hbytes, hrange, int_from_bytes, int_to_bytes
-from hypothesis.internal.conjecture.data import Overrun, Status
+from hypothesis.internal.conjecture.data import ConjectureResult, Overrun, Status
 from hypothesis.internal.conjecture.floats import (
     DRAW_FLOAT_LABEL,
     float_to_lex,
@@ -274,20 +274,11 @@ class Shrinker(object):
         """
         self.__engine = engine
         self.__predicate = predicate
-        self.discarding_failed = False
         self.__shrinking_prefixes = set()
         self.__derived_values = {}
 
         self.initial_size = len(initial.buffer)
 
-        # We add a second level of caching local to the shrinker. This is a bit
-        # of a hack. Ideally we'd be able to rely on the engine's functionality
-        # for this. Doing it this way has two benefits: Firstly, the engine
-        # does not currently cache overruns (and probably shouldn't, but could
-        # recreate them on demand if necessary), and secondly Python dicts are
-        # much faster than our pure Python tree-based lookups.
-        self.__test_function_cache = {}
-
         # We keep track of the current best example on the shrink_target
         # attribute.
         self.shrink_target = None
@@ -296,7 +287,6 @@ class Shrinker(object):
 
         self.initial_calls = self.__engine.call_count
 
-        self.current_pass_depth = 0
         self.passes_by_name = {}
         self.clear_passes()
 
@@ -593,8 +583,10 @@ class Shrinker(object):
     def examples(self):
         return self.shrink_target.examples
 
-    def all_block_bounds(self):
-        return self.shrink_target.all_block_bounds()
+    def all_block_bounds(self, target=None):
+        if target is None:
+            target = self.shrink_target
+        return [(b.start, b.end) for b in target.blocks]
 
     @derived_value
     def examples_by_label(self):
@@ -714,7 +706,7 @@ class Shrinker(object):
 
         current = self.shrink_target
 
-        blocked = [current.buffer[u:v] for u, v in current.all_block_bounds()]
+        blocked = [current.buffer[u:v] for u, v in self.all_block_bounds(current)]
 
         changed = [
             i
@@ -759,7 +751,7 @@ class Shrinker(object):
         self.__changed_blocks.add(i)
 
     def update_shrink_target(self, new_target):
-        assert new_target.frozen
+        assert isinstance(new_target, ConjectureResult)
         if self.shrink_target is not None:
             current = self.shrink_target.buffer
             new = new_target.buffer
@@ -767,7 +759,7 @@ class Shrinker(object):
             self.shrinks += 1
             if (
                 len(new_target.blocks) != len(self.shrink_target.blocks)
-                or new_target.all_block_bounds() != self.all_block_bounds()
+                or self.all_block_bounds(new_target) != self.all_block_bounds()
             ):
                 self.clear_change_tracking()
             else:
